their families.
no anxiety about providing the means of subsistence for themselves and
which should live in ease, happiness, and comparative leisure; and feel
decisive against the possible existence of a society, all the members of
of it even for a single century. And it appears, therefore, to be
agrarian regulations in their utmost extent, could remove the pressure
of this law which pervades all animated nature. No fancied equality, no
comparison of this. I see no way by which man can escape from the weight
All other arguments are of slight and subordinate consideration in
me appears insurmountable in the way to the perfectibility of society.
constantly keep their effects equal, form the great difficulty that to
production in the earth, and that great law of our nature which must
This natural inequality of the two powers of population and of
